--[[====================================================================================
Keybindings & MIDI Control to set start and end loop points of any size, spanning across
multiple patterns if desired.

Author: Dale Powell (Kazakore) <dj_kaza@hotmail.com>
Version: 0.2
====================================================================================]]--

local rns = renoise.song
local tool = renoise.tool()

-- Set Loop Selection
  local function loopsel()
    local s = rns().transport.edit_pos
    local e = rns().transport.edit_pos
    s.line = rns().selection_in_pattern.start_line
    e.line = rns().selection_in_pattern.end_line
    rns().transport.loop_range = {s, e}
  end
  
-- Set Loop Start
  local function setstart()
    rns().transport.loop_range = {rns().transport.edit_pos, rns().transport.loop_range[2]}
  end
  
-- Set Loop End
  local function setend()
    rns().transport.loop_range = {rns().transport.loop_range[1], rns().transport.edit_pos}
  end
  
-- Clear Loop
  local function clearloop()
    local s = rns().transport.edit_pos
    local e = rns().transport.edit_pos
    s.sequence = 1
    s.line = 1
    e.sequence = #rns().sequencer.pattern_sequence
    e.line = rns().patterns[renoise.song().sequencer.pattern_sequence[#rns().sequencer.pattern_sequence]].number_of_lines +1
    rns().transport.loop_range = {s, e}
  end
  
-- Keybindings
  tool:add_keybinding {
    name = "Pattern Editor:Transition Loop Tool:Set Loop Start",
    invoke = function() setstart() end
  }
  tool:add_keybinding {
    name = "Pattern Editor:Transition Loop Tool:Set Loop End",
    invoke = function() setend() end
  }
  tool:add_keybinding {
    name = "Pattern Editor:Transition Loop Tool:Loop Selection",
    invoke = function() loopsel() end
  }
  tool:add_keybinding {
    name = "Pattern Editor:Transition Loop Tool:Clear Loop",
    invoke = function() clearloop() end
  }
  
-- MIDI Mapping
  tool:add_midi_mapping {
    name = "Tools:Transition Loop Tool:Set Loop Start",
    invoke = function(message)
      if (message:is_trigger()) then
        setstart()
      end
    end
  } 
  tool:add_midi_mapping {
    name = "Tools:Transition Loop Tool:Set Loop End",
    invoke = function(message)
      if (message:is_trigger()) then
        setend()
      end
    end
  } 
  tool:add_midi_mapping {
    name = "Tools:Transition Loop Tool:Loop Selection",
    invoke = function(message)
      if (message:is_trigger()) then
        loopsel()
      end
    end
  } 
  tool:add_midi_mapping {
    name = "Tools:Transition Loop Tool:Clear Loop",
    invoke = function(message)
      if (message:is_trigger()) then
        clearloop()
      end
    end
  } 
